% file zoomJGL/man/zoomJGL.Rd
% copyright (C) Eugen Pircalabelu
%
\name{zoomJGL}
\alias{zoomJGL}
\title{
Estimate zoom-in/out directed and undirected graphs at different coarsness scales
}
\description{
Estimate jointly multiple graphs that resemble each other.
}
\usage{
zoomJGL(Y,lambda1,lambda2,list.controls)
}
\arguments{
\item{Y}{
List of data matrices where each element of the list is a matrix of dimension Txp, with p increasing across scales.
}
\item{lambda1}{
Regularization parameter that dictates the sparsity of the undirected graphs
}
\item{lambda2}{
Regularization parameter that dictates the sparsity of the directed graphs
}
\item{list.controls}{
List of control parameters specifying

\code{model}: The autoregressive model; default AR1

\code{penalize.diagonal}: Indicator if the diagonal should be penalized or not; default FALSE

\code{kstar}: The starting scale; default 1 (ie. performs a zoom-in)

\code{maxiter}: Number of maximal iterations ; default 200

\code{tol}: Tolerance threshold for assesing convergence; default 1e-02

\code{rho}: Parameter used for the ADMM optimizer; default 1

\code{rho.increments}: Parameter used for the ADMM optimizer; default 1

\code{alpha}: Parameter used for the ADMM optimizer; default 0.7

}
}

\value{
A list containing the objects:

\item{Theta}{
List of parameter matrices of size pxp at each scale. The parameters correspond to undirected edges in the graphs. 
}
\item{Gamma}{
List of parameter matrices of size pxp at each scale. The parameters correspond to directed edges in the graphs. 
}

\item{Undirected}{
List of adjacency matrices of size pxp at each scale corresponding to the undirected graphs. 
}
\item{Directed}{
List of adjacency matrices of size pxp at each scale corresponding to the undirected graphs. 
}

\item{Constraint1}{
List of matrices that map lambda1 to enforce sparsity and a similar structure across scales for the undirected graphs. 
}
\item{Constraint2}{
List of matrices that map lambda1 to enforce sparsity and a similar structure across scales for the undirected graphs. 
}
\item{Convergence}{
Convergence performance at each iteration of the algorithm. The values for DiffObjective, DiffZTheta, DiffGamma which capture the difference between 
two consecutive iterations for the objective function, the matrices Z-Theta and Gamma should get closer to 0 across iterations.     
}

\item{IC}{
Information criteria scores: AIC, BIC, eBIC(.1), eBIC(.5), eBIC(.9)     
}
 

}
\seealso{
\code{\link{}}
}

\examples{
data(fmri)
obj=zoomJGL(Y,.4,.2)
}
\keyword{algebra}
