// -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-

// we only include RcppEigen.h which pulls Rcpp.h in for us
#include <RcppEigen.h>
#include "linear_alg_exports.h"
#include <cstdlib>
#include <iostream>
#include <R.h>

using namespace Rcpp;
using namespace Eigen;
using namespace std;


void Lasso ( const MatrixXd& CX, const MatrixXd& CXXZ, const MatrixXd& Z,
 int p, int q, int warm, const MatrixXd& lamin, const MatrixXd& Bout, MatrixXd& out)
{      
	 double J, H, AH, tmp,bnew;
	 int r,c,k,j;
	 MatrixXd B	= MatrixXd::Zero(p,q);
	
	if( warm == 1) B=Bout;
	
	for(r=0; r < p; r++)
    {
      for(c=0; c <q; c++)
      {
		J=0;  
        if(lamin(r,c)<100000)
		{
		
        for(j=0; j <p; j++)
        {
          for(k=0; k <q; k++)
          {
            if(B(j,k) != 0)
            {
              J+=B(j,k)*CX(r,j)*Z(k,c);  
            }
          }
        }
		H = B(r,c) + (CXXZ(r,c)-J)/(CX(r,r)*Z(c,c));
        AH = fabs(H);
        tmp =AH - lamin(r,c)/(CX(r,r)*Z(c,c));
        bnew=0;
        if(tmp  > 0 )
        {
          if(H > 0)
          {
            bnew = tmp;
          }
          if( H < 0 )
          {
            bnew = -tmp;
          }
        }
        B(r,c)=bnew;
		}
	  }
    }
	out = B;
 }

 void Lasso2 ( const MatrixXd& CX, const MatrixXd& CXXZ, const MatrixXd& Z,
 int p, int q, int warm, const MatrixXd& lamin, const MatrixXd& Bout, MatrixXd& out)
{      
	 double  H, AH, tmp,bnew;
	 int r,c;
	 MatrixXd B	= MatrixXd::Zero(p,q);
     MatrixXd ee= MatrixXd::Zero(p,q);
	
	if( warm == 1) B=Bout;
	
	for(r=0; r < p; r++)
    {
      for(c=0; c <q; c++)
      {
       ee=CX*B*Z;
	   H=B(r,c) + (CXXZ(r,c)-ee(r,c))/(CX(r,r)*Z(c,c));
	   AH = fabs(H);
        tmp =AH - lamin(r,c)/(CX(r,r)*Z(c,c));
        bnew=0;
        if(tmp  > 0 )
        {
          if(H > 0)
          {
            bnew = tmp;
          }
          if( H < 0 )
          {
            bnew = -tmp;
          }
        }
        B(r,c)=bnew;
		}
	  }
	out = B;
 }
 
SEXP R_blassoEigen (SEXP R_CX,  SEXP R_CXXZ, SEXP R_Z, SEXP R_pin, SEXP R_qin, SEXP R_lamin,
					 SEXP R_Bout,  SEXP R_warm) 
{
	NumericMatrix Rcpp_CX(R_CX);	
	const int i1 = Rcpp_CX.nrow(), j1 = Rcpp_CX.ncol();
	Map<MatrixXd> CX(Rcpp_CX.begin(), i1 ,j1);
	
	NumericMatrix Rcpp_CXXZ(R_CXXZ);	
	const int i2 = Rcpp_CXXZ.nrow(), j2 = Rcpp_CXXZ.ncol();
	Map<MatrixXd> CXXZ(Rcpp_CXXZ.begin(), i2 ,j2);
	
	NumericMatrix Rcpp_Z(R_Z);	
	const int i3 = Rcpp_Z.nrow(), j3 = Rcpp_Z.ncol();
	Map<MatrixXd> Z(Rcpp_Z.begin(), i3 ,j3);
	
		
	int p = as<int>(R_pin); 
	int q = as<int>(R_qin); 
	int warm = as<int>(R_warm);
	
	NumericMatrix Rcpp_lamin(R_lamin);	
	const  int i4 = Rcpp_lamin.nrow(), j4 = Rcpp_lamin.ncol();
	Map<MatrixXd> lamin(Rcpp_lamin.begin(), i4 ,j4);
	
    NumericMatrix Rcpp_Bout(R_Bout);	
    const int i5 = Rcpp_Bout.nrow(), j5 = Rcpp_Bout.ncol();
    Map<MatrixXd> Bout(Rcpp_Bout.begin(), i5 ,j5);
	
	MatrixXd out; 
	
	Lasso2(CX, CXXZ, Z, p,q,warm, lamin, Bout, out);
	 return List::create(Named("Gamma") = out);       
}







 SEXP R_eigen_decom(SEXP R_A)
 {
	 NumericMatrix Rcpp_A(R_A);	
	 const int i1 = Rcpp_A.nrow(), j1 = Rcpp_A.ncol();
	 Map<MatrixXd> A(Rcpp_A.begin(), i1 ,j1);
    // AB =eigen_decom(A);
	
   const  EigenSolver<MatrixXd> es(A);
   return List::create(Named("L") = es.eigenvalues());
   }
   
  
  SEXP R_eigen_decom2(SEXP R_A)
 {
	 NumericMatrix Rcpp_A(R_A);	
	 const int i1 = Rcpp_A.nrow(), j1 = Rcpp_A.ncol();
	 Map<MatrixXd> A(Rcpp_A.begin(), i1 ,j1);
 //   AB =eigen_decom(A);
	
  const  SelfAdjointEigenSolver<MatrixXd> eigensolver(A);
  return List::create(Named("values") = eigensolver.eigenvalues(),
  Named("vectors") = eigensolver.eigenvectors());
  }
 
  SEXP R_xytx(SEXP R_x,SEXP R_y)
 {
	NumericMatrix Rcpp_x(R_x);	
	 const int i1 = Rcpp_x.nrow(), j1 = Rcpp_x.ncol();
	 Map<MatrixXd> x(Rcpp_x.begin(), i1 ,j1); 
 
 	NumericMatrix Rcpp_y(R_y);	
	 const int i2 = Rcpp_y.nrow(), j2 = Rcpp_y.ncol();
	 Map<MatrixXd> y(Rcpp_y.begin(), i2 ,j2); 
	
	MatrixXd op = x * y *x.transpose();
	return List::create(Named("AB") = op);       
}

  SEXP R_xy(SEXP R_x,SEXP R_y)
 {
	NumericMatrix Rcpp_x(R_x);	
	 const int i1 = Rcpp_x.nrow(), j1 = Rcpp_x.ncol();
	 Map<MatrixXd> x(Rcpp_x.begin(), i1 ,j1); 
 
 	NumericMatrix Rcpp_y(R_y);	
	 const int i2 = Rcpp_y.nrow(), j2 = Rcpp_y.ncol();
	 Map<MatrixXd> y(Rcpp_y.begin(), i2 ,j2); 
	
	MatrixXd op = x * y; 
	return List::create(Named("XY") = op);       
}



  SEXP R_xty(SEXP R_x,SEXP R_y)
 {
	NumericMatrix Rcpp_x(R_x);	
	 const int i1 = Rcpp_x.nrow(), j1 = Rcpp_x.ncol();
	 Map<MatrixXd> x(Rcpp_x.begin(), i1 ,j1); 
 
 	NumericMatrix Rcpp_y(R_y);	
	 const int i2 = Rcpp_y.nrow(), j2 = Rcpp_y.ncol();
	 Map<MatrixXd> y(Rcpp_y.begin(), i2 ,j2); 
	
	MatrixXd op = x * y.transpose(); 
	return List::create(Named("XtY") = op);       
}

  SEXP R_txy(SEXP R_x,SEXP R_y)
 {
	NumericMatrix Rcpp_x(R_x);	
	 const int i1 = Rcpp_x.nrow(), j1 = Rcpp_x.ncol();
	 Map<MatrixXd> x(Rcpp_x.begin(), i1 ,j1); 
 
 	NumericMatrix Rcpp_y(R_y);	
	 const int i2 = Rcpp_y.nrow(), j2 = Rcpp_y.ncol();
	 Map<MatrixXd> y(Rcpp_y.begin(), i2 ,j2); 
	
	MatrixXd op = x.transpose() * y; 
	return List::create(Named("tXY") = op);       
}
  SEXP R_xyz(SEXP R_x,SEXP R_y,SEXP R_z)
 {
	NumericMatrix Rcpp_x(R_x);	
	 const int i1 = Rcpp_x.nrow(), j1 = Rcpp_x.ncol();
	 Map<MatrixXd> x(Rcpp_x.begin(), i1 ,j1); 
 
 	NumericMatrix Rcpp_y(R_y);	
	 const int i2 = Rcpp_y.nrow(), j2 = Rcpp_y.ncol();
	 Map<MatrixXd> y(Rcpp_y.begin(), i2 ,j2); 
	
	NumericMatrix Rcpp_z(R_z);	
	 const int i3 = Rcpp_z.nrow(), j3 = Rcpp_y.ncol();
	 Map<MatrixXd> z(Rcpp_z.begin(), i3 ,j3); 
	 
	MatrixXd op = x * y * z;
	return List::create(Named("XYZ") = op);       
}
 
  SEXP R_xytz(SEXP R_x,SEXP R_y,SEXP R_z)
 {
	NumericMatrix Rcpp_x(R_x);	
	 const int i1 = Rcpp_x.nrow(), j1 = Rcpp_x.ncol();
	 Map<MatrixXd> x(Rcpp_x.begin(), i1 ,j1); 
 
 	NumericMatrix Rcpp_y(R_y);	
	 const int i2 = Rcpp_y.nrow(), j2 = Rcpp_y.ncol();
	 Map<MatrixXd> y(Rcpp_y.begin(), i2 ,j2); 
	
	NumericMatrix Rcpp_z(R_z);	
	 const int i3 = Rcpp_z.nrow(), j3 = Rcpp_y.ncol();
	 Map<MatrixXd> z(Rcpp_z.begin(), i3 ,j3); 
	 
	MatrixXd op = x * y * z.transpose();
	return List::create(Named("XYtZ") = op);       
}

  SEXP R_txyz(SEXP R_x,SEXP R_y,SEXP R_z)
 {
	NumericMatrix Rcpp_x(R_x);	
	 const int i1 = Rcpp_x.nrow(), j1 = Rcpp_x.ncol();
	 Map<MatrixXd> x(Rcpp_x.begin(), i1 ,j1); 
 
 	NumericMatrix Rcpp_y(R_y);	
	 const int i2 = Rcpp_y.nrow(), j2 = Rcpp_y.ncol();
	 Map<MatrixXd> y(Rcpp_y.begin(), i2 ,j2); 
	
	NumericMatrix Rcpp_z(R_z);	
	 const int i3 = Rcpp_z.nrow(), j3 = Rcpp_y.ncol();
	 Map<MatrixXd> z(Rcpp_z.begin(), i3 ,j3); 
	 
	MatrixXd op = x.transpose() * y * z;
	return List::create(Named("tXYZ") = op);       
}

  SEXP R_elemwisexy(SEXP R_x,SEXP R_y)
 {
	NumericMatrix Rcpp_x(R_x);	
	 const int i1 = Rcpp_x.nrow(), j1 = Rcpp_x.ncol();
	 Map<MatrixXd> x(Rcpp_x.begin(), i1 ,j1); 
 
 	NumericMatrix Rcpp_y(R_y);	
	 const int i2 = Rcpp_y.nrow(), j2 = Rcpp_y.ncol();
	 Map<MatrixXd> y(Rcpp_y.begin(), i2 ,j2); 
	
	MatrixXd op = x.cwiseProduct(y);
	return List::create(Named("elemwiseXY") = op);       
}



// t(A)*A
MatrixXd matrix_crossproduct_single(const MatrixXd& x)
{
	int n(x.cols());
	MatrixXd tAA(MatrixXd(n, n).setZero().selfadjointView<Lower>().rankUpdate(x.adjoint()));
	return(tAA);
}

SEXP R_txx(SEXP R_x) 
{
	 NumericMatrix Rcpp_x(R_x);	
	 const int i1 = Rcpp_x.nrow(), j1 = Rcpp_x.ncol();
	 Map<MatrixXd> x(Rcpp_x.begin(), i1 ,j1); 

	MatrixXd	txx=matrix_crossproduct_single (x);
	return List::create(Named("tXX") = txx);       
}

// A*t(A)
MatrixXd matrix_tcrossproduct_single(const MatrixXd& x)
{
	int m(x.rows());
	MatrixXd AAt(MatrixXd(m, m).setZero().selfadjointView<Lower>().rankUpdate(x));
	return(AAt);
}

SEXP R_xtx(SEXP R_x)
{
	NumericMatrix Rcpp_x(R_x);	
	 const int i1 = Rcpp_x.nrow(), j1 = Rcpp_x.ncol();
	 Map<MatrixXd> x(Rcpp_x.begin(), i1 ,j1); 
 
	MatrixXd	xtx=matrix_tcrossproduct_single (x);
	return List::create(Named("XtX") = xtx);       
}

// A^{-1}
MatrixXd matrix_inverse(const MatrixXd& A)
{
	MatrixXd Ainv;
	Ainv			 		  = A.inverse();
	return(Ainv);
}

SEXP R_matrix_inverse(SEXP R_A) 
{
	NumericMatrix Rcpp_A(R_A);	
	const int i1 = Rcpp_A.nrow(), j1 = Rcpp_A.ncol();
	Map<MatrixXd> A(Rcpp_A.begin(), i1 ,j1);
	
	MatrixXd	Ainv=matrix_inverse(A);
	return List::create(Named("Ainv") = Ainv);       
}



Eigen::MatrixXd rcppeigen_hello_world() {
    Eigen::MatrixXd m1 = Eigen::MatrixXd::Identity(3, 3);
    Eigen::MatrixXd m2 = Eigen::MatrixXd::Random(3, 3);
	                     
    return m1 + 3 * (m1 + m2);
}


// another simple example: outer product of a vector, 
// returning a matrix
//
// [[Rcpp::export]]
Eigen::MatrixXd rcppeigen_outerproduct(const Eigen::VectorXd & x) {
    Eigen::MatrixXd m = x * x.transpose();
    return m;
}

// and the inner product returns a scalar
//
// [[Rcpp::export]]
double rcppeigen_innerproduct(const Eigen::VectorXd & x) {
    double v = x.transpose() * x;
    return v;
}

// and we can use Rcpp::List to return both at the same time
//
// [[Rcpp::export]]
Rcpp::List rcppeigen_bothproducts(const Eigen::VectorXd & x) {
    Eigen::MatrixXd op = x * x.transpose();
    double          ip = x.transpose() * x;
    return Rcpp::List::create(Rcpp::Named("outer")=op,
                              Rcpp::Named("inner")=ip);
}


